# Code to illustrate the random walk property in 
# Challet et al, Naturwissenschaften 2005 (in press)
# http://dx.doi.org/10.1007/s00114-005-0001-1
# In case of any questions please contact jost<at>cict.fr

# this is exactly the same code as in 'RandoWalkWithTrail-graph.R', but 
# iterated in a Monte Carlo loop to compute mean path length and expected 
# mean path legnth in the case of a diffusive random walk (lpmPist=0).
# For lpmPiste > 0 the computed mean path length will be significantly different
# from this expected length if numOfMC is sufficiently high.

# Execute this script by starting R (http://www.r-project.org), point its
# working directory to the one containing this file, then type
# > source("RandomWalkWithTrail-MC.R")

# play around with lpmPiste (trail following) and numOfMC (precision of mean
# path length).


getCosAngle = function() {
  y = runif(1);
  return(asin(2*y-1));
}

getRandExp = function( lambda ) {
  x = runif(1);
  if (is.finite(lambda)) { 
      return(-log(x)/lambda); 
  } else {
      return(0);
  }
}

xmax = 20;
xmin = -xmax;
ymax = 5;
ymin = -ymax;
trajPred = pi*2*xmax*2*ymax/(4*(xmax+ymax));
lpmH = 1; 
lambdaH=1/lpmH; 
lpmB = 2; 
lambdaB=1/lpmB; 
##################
lpmPiste = 0; # choose lpmPiste > 0 to obtain trail following and thus
# non-diffusive random walks
lambdaPiste = 1/lpmPiste;
sdFoncPhase = 1.2; 
propEnBas = xmax/(xmax+ymax);
distCum = 0;
distCarreCum = 0;
##################
# increase this value to get better precision of mean path length
numOfMC = 1000;

for (i in 1:numOfMC) {
if (runif(1) < propEnBas) {
    if (runif(1) < 0.5) { 
      xact = 2*xmax*runif(1) - xmax;
      yact = -ymax;
      phi = getCosAngle()+pi/2;
  } else {               
      xact = 2*xmax*runif(1) - xmax;
      yact = ymax;
      phi = getCosAngle()-pi/2;
}
} else {
  xact = -xmax;
  yact = 2*ymax*runif(1) - ymax;
  phi = getCosAngle();
}
dist = 0.0;
if (yact < 0.0) {lpNext = getRandExp(lambdaB);} else {
 lpNext = getRandExp(lambdaH); }
gotOut = F;
while (((abs(xact + lpNext*cos(phi)) < xmax) | (sign(yact*(yact + lpNext*sin(phi))) < 0)) &
       ((abs(yact + lpNext*sin(phi)) < ymax) | (sign(yact*(yact + lpNext*sin(phi))) < 0)) &
       gotOut == F) {
 if ( sign(yact*(yact + lpNext*sin(phi))) >= 0 ) { 
   xnew = xact + lpNext*cos(phi);
   ynew = yact + lpNext*sin(phi);
   dist = dist + sqrt((xnew-xact)^2+(ynew-yact)^2);
   xact = xnew;
   yact = ynew;
   phi =  phi + rnorm(1,0,sdFoncPhase);
   if (yact < 0.0) {lpNext = getRandExp(lambdaB);} else {
     lpNext = getRandExp(lambdaH); }
 } else { 
   facteur = abs(yact/(lpNext*sin(phi)));
   if (abs(xact+facteur*lpNext*cos(phi)) > xmax) { 
       facteur = facteur*(xmax-abs(xact))/(abs(facteur*lpNext*cos(phi)));
       gotOut=T;}
   xnew = xact + facteur*lpNext*cos(phi);
   ynew = yact + facteur*lpNext*sin(phi);
   dist = dist + sqrt((xnew-xact)^2+(ynew-yact)^2);
   xact = xnew;
   if (gotOut) {
       yact = ynew ;
   } else {
       yact = 0.0; 
   }
   phiRel = phi/(2*pi) - round(phi/(2*pi));
   phiVieux = phi;
   if (abs(phiRel) < 0.25)  { 
     phi = 0; 
   } else { 
     phi = pi;
   }

   if (!gotOut) {
     lpPiste = getRandExp(lambdaPiste);
     if (abs(xact + lpPiste*cos(phi)) < xmax ) { 
       xnew = xact + lpPiste*cos(phi);
       dist = dist + (xnew-xact)^2;
       xact = xnew;
       phi = phiVieux; 
       if (phi/(2*pi) - round(phi/(2*pi)) > 0.0) { 
  	 lpNext = getRandExp(lambdaH);
       } else {
           lpNext = getRandExp(lambdaB);
       }
     } else {
       lpNext = lpPiste;
       gotOut = T;
     }
   }
 } 
}
if (abs(xact) < xmax | sign(xact)*sign(cos(phi)) < 0 ) {
  xVir = xact + lpNext*cos(phi);
  factX = (min(abs(xVir),xmax)-sign(xact*xVir)*abs(xact))/
             (abs(xVir) - sign(xact*xVir)* abs(xact));
  if (gotOut == T) {
      factY = 1;
  } else {
      yVir = yact + lpNext*sin(phi);
      factY = (min(abs(yVir),ymax)- sign(yact*yVir)*abs(yact))/
               (abs(yVir) - sign(yact*yVir)* abs(yact));
  }
  facteur = min(factX,factY)
  xnew = xact + facteur*lpNext*cos(phi);
  ynew = yact + facteur*lpNext*sin(phi);
  dist = dist + sqrt((xnew-xact)^2+(ynew-yact)^2);
}
   
# update cumulated path length and cumulated squared path length
 if (is.nan(dist)) print("dist is nan"); # for debugging, should never occur
 distCum = dist + distCum;
 distCarreCum = dist^2 + distCarreCum;
}

# final simulation results
distMean = distCum/numOfMC; # mean path length
distCarreMean = distCarreCum/numOfMC;
distVar = distCarreMean - distMean^2; # variance of path length
distSE = sqrt(distVar/numOfMC); #standard error of path length
print(sprintf("Predicted path length if lpmPiste=0 : %f",trajPred))
print(sprintf("Path length = %f +- %f",distMean,distSE))
print(sprintf("Confidence intervale at 0.95 = (%f,%f)",distMean-1.96*distSE,
               distMean+1.96*distSE));
               