# Code to illustrate the random walk property in 
# Challet et al, Naturwissenschaften 2005 (in press)
# http://dx.doi.org/10.1007/s00114-005-0001-1
# In case of any questions please contact jost<at>cict.fr

# This program simulates a diffusive random walk in a rectangle centered around 
# 0 and with dimensions (xmin,xmax) and (ymin,ymax). 
# The random walk is characterised by a normal shape phase function with sd
# sdFoncPhase and exponentially distributed free paths with mean length
# lpm. lpm changes between different parts of the rectangle, having
# value lpmH in the upper half, lpmB in the lower half and
# lpmPiste on the trail between the first two (it may be set to 0 to have 
# a pure diffusive random walk).
# Individuals enter the rectangle from either below or the left with an
# incidence distributed as a cosine (as it would in standard diffusion).
#
# This script only computes one path and plots it to give you an idea how
# the random walk looks like. To run the script simply start R, 
# (http://www.r-project.org)
# make sure its working directory points to the one containing this 
# file, then type
# > source("RandomWalkWithTrail-graph.R")
# 
# To compute the mean path length use the script 
# 'RandomWalkWithTrail-MC.R'
# that does a Monte Carlo simulation (without graphs to speed up) to
# compute mean path length and standard error.

getCosAngle = function() {
  # if y in (0,1), then x = arcsin(2*y-1) and distributed 
  # as a cosine distribution with mean 0 and values between -pi/2 et pi/2
  y = runif(1);
  return(asin(2*y-1));
}

getRandExp = function( lambda ) {
  # lambda is the 'rate' (per unit distance) to change direction, 1/lambda the  mean 
  # straight walk length
  x = runif(1);
  if (is.finite(lambda)) { 
      return(-log(x)/lambda); 
  } else {
      return(0);
  }
}

# definition of the rectangle
xmax = 20;
xmin = -xmax;
ymax = 5;
ymin = -ymax;
trajPred = pi*2*xmax*2*ymax/(4*(xmax+ymax));
lpmH = 1; # mean free path in upper part of rectangle
lambdaH=1/lpmH; # rate to change direction
lpmB = 2; # lower part of rectangle
lambdaB=1/lpmB; 
lpmPiste = 5; # mean free path on central trail (may be set to 0)
lambdaPiste = 1/lpmPiste;
sdFoncPhase = 1.2; # this sd corresponds to g = 0.43 (mean cosine of phase 
# function

plot(c(xmin,xmin,xmax,xmax,xmin),c(ymin,ymax,ymax,ymin,ymin),type='l');
# computing entry point of the individual
propEnBas = xmax/(xmax+ymax); # probability to enter from below or top
if (runif(1) < propEnBas) { 
    if (runif(1) < 0.5) { # enter from below
      xact = 2*xmax*runif(1) - xmax;
      yact = -ymax;
      phi = getCosAngle()+pi/2;
  } else {               # enter from the top
      xact = 2*xmax*runif(1) - xmax;
      yact = ymax;
      phi = getCosAngle()-pi/2;
}
} else { # enter from the left
  xact = -xmax;
  yact = 2*ymax*runif(1) - ymax;
  phi = getCosAngle();
}
points(xact,yact)

dist = 0.0;
if (yact < 0.0) {lpNext = getRandExp(lambdaB);} else {
 lpNext = getRandExp(lambdaH); }
gotOut = F;
while (((abs(xact + lpNext*cos(phi)) < xmax) | (sign(yact*(yact + lpNext*sin(phi))) < 0)) &
       ((abs(yact + lpNext*sin(phi)) < ymax) | (sign(yact*(yact + lpNext*sin(phi))) < 0)) &
       gotOut == F) {
 if ( sign(yact*(yact + lpNext*sin(phi))) >= 0 ) { # trail has not been crossed
   xnew = xact + lpNext*cos(phi);
   ynew = yact + lpNext*sin(phi);
   dist = dist + sqrt((xnew-xact)^2+(ynew-yact)^2);
   lines(c(xact,xnew),c(yact,ynew));
   xact = xnew;
   yact = ynew;
   phi =  phi + rnorm(1,0,sdFoncPhase);
   if (yact < 0.0) {lpNext = getRandExp(lambdaB);} else {
     lpNext = getRandExp(lambdaH); }
 } else { # the trail has been crossed
   facteur = abs(yact/(lpNext*sin(phi)));
   if (abs(xact+facteur*lpNext*cos(phi)) > xmax) { 
       facteur = facteur*(xmax-abs(xact))/(abs(facteur*lpNext*cos(phi)));
       gotOut=T;}
   xnew = xact + facteur*lpNext*cos(phi);
   ynew = yact + facteur*lpNext*sin(phi);
   dist = dist + sqrt((xnew-xact)^2+(ynew-yact)^2);
   lines(c(xact,xnew),c(yact,ynew));
   points(xnew,ynew,col='red');
   xact = xnew;
   if (gotOut) {
       yact = ynew ;
   } else {
       yact = 0.0; 
   }
   phiRel = phi/(2*pi) - round(phi/(2*pi));
   phiVieux = phi;
   if (abs(phiRel) < 0.25)  { # continue to the left
     phi = 0; 
   } else { # continue to the right
     phi = pi;
   }
   # we are now on the trail
   if (!gotOut) {
     lpPiste = getRandExp(lambdaPiste);
     if (abs(xact + lpPiste*cos(phi)) < xmax ) { # we stay inside the rectangle
       xnew = xact + lpPiste*cos(phi);
       dist = dist + (xnew-xact)^2;
       points(xnew,yact); lines(c(xact,xnew),c(yact,yact),col='red');
       xact = xnew;
       phi = phiVieux; # we continue in the same direction we entered the trail
       if (phi/(2*pi) - round(phi/(2*pi)) > 0.0) { 
  	 lpNext = getRandExp(lambdaH);
       } else {
           lpNext = getRandExp(lambdaB);
       }
     } else {
       lpNext = lpPiste;
       gotOut = T;
     }
   }
 } 
}
if (abs(xact) < xmax | sign(xact)*sign(cos(phi)) < 0 ) {
  lines(c(xact,xact + lpNext*cos(phi)),c(yact,yact+lpNext*sin(phi)),col='blue')
  xVir = xact + lpNext*cos(phi);
  factX = (min(abs(xVir),xmax)-sign(xact*xVir)*abs(xact))/
             (abs(xVir) - sign(xact*xVir)* abs(xact));
  if (gotOut == T) {
      factY = 1;
  } else {
      yVir = yact + lpNext*sin(phi);
      factY = (min(abs(yVir),ymax)- sign(yact*yVir)*abs(yact))/
               (abs(yVir) - sign(yact*yVir)* abs(yact));
  }
  facteur = min(factX,factY)
  xnew = xact + facteur*lpNext*cos(phi);
  ynew = yact + facteur*lpNext*sin(phi);
  dist = dist + sqrt((xnew-xact)^2+(ynew-yact)^2);
  lines(c(xact,xnew),c(yact,ynew));
  points(xnew,ynew);
}
# print the path distance
print(dist);

